@@ -31,6 +31,8 @@
 
 #include <osmocom/hnbgw/vty.h>
 
+#include <osmocom/hnbgw/hnb.h>
+#include <osmocom/hnbgw/hnb_persistent.h>
 #include <osmocom/hnbgw/hnbgw.h>
 #include <osmocom/hnbgw/hnbgw_cn.h>
 #include <osmocom/hnbgw/context_map.h>
@@ -84,7 +86,7 @@ DEFUN(cfg_hnbgw_iucs, cfg_hnbgw_iucs_cmd,
       "iucs", "Configure IuCS options")
 {
 	vty->node = IUCS_NODE;
-	vty->index = &g_hnbgw->sccp.cnpool_iucs;
+	vty->index = g_hnbgw->sccp.cnpool_iucs;
 	return CMD_SUCCESS;
 }
 
@@ -98,7 +100,7 @@ DEFUN(cfg_hnbgw_iups, cfg_hnbgw_iups_cmd,
       "iups", "Configure IuPS options")
 {
 	vty->node = IUPS_NODE;
-	vty->index = &g_hnbgw->sccp.cnpool_iups;
+	vty->index = g_hnbgw->sccp.cnpool_iups;
 	return CMD_SUCCESS;
 }
 
@@ -123,7 +125,6 @@ int hnbgw_vty_go_parent(struct vty *vty)
 
 static void _show_cnlink(struct vty *vty, struct hnbgw_cnlink *cnlink)
 {
-	struct osmo_ss7_route *rt;
 	struct osmo_ss7_instance *ss7;
 
 	if (!cnlink) {
@@ -147,16 +148,15 @@ static void _show_cnlink(struct vty *vty, struct hnbgw_cnlink *cnlink)
 		return;
 	}
 
-	vty_out(vty, "%s <->",
+	vty_out(vty, "%s: %s <->",
+		cnlink->name,
 		osmo_sccp_user_name(cnlink->hnbgw_sccp_user->sccp_user));
 	vty_out(vty, " %s%s%s%s",
 		cnlink->use.remote_addr_name ? : "",
 		cnlink->use.remote_addr_name ? "=" : "",
-		cnlink_sccp_addr_to_str(cnlink, &cnlink->remote_addr),
+		hnbgw_cnlink_sccp_addr_to_str(cnlink, &cnlink->remote_addr),
 		VTY_NEWLINE);
 
-	rt = osmo_ss7_route_lookup(ss7, cnlink->remote_addr.pc);
-	vty_out(vty, "      SS7 route: %s%s", osmo_ss7_route_name(rt, true), VTY_NEWLINE);
 	vty_out(vty, "      RANAP state: %s%s", osmo_fsm_inst_state_name(cnlink->fi), VTY_NEWLINE);
 }
 
@@ -165,26 +165,14 @@ DEFUN(show_cnlink, show_cnlink_cmd, "show cnlink",
 {
 	struct hnbgw_cnlink *cnlink;
 	vty_out(vty, "IuCS: ");
-	llist_for_each_entry(cnlink, &g_hnbgw->sccp.cnpool_iucs.cnlinks, entry)
+	llist_for_each_entry(cnlink, &g_hnbgw->sccp.cnpool_iucs->cnlinks, entry)
 		_show_cnlink(vty, cnlink);
 	vty_out(vty, "IuPS: ");
-	llist_for_each_entry(cnlink, &g_hnbgw->sccp.cnpool_iups.cnlinks, entry)
+	llist_for_each_entry(cnlink, &g_hnbgw->sccp.cnpool_iups->cnlinks, entry)
 		_show_cnlink(vty, cnlink);
 	return CMD_SUCCESS;
 }
 
-static void vty_out_ofd_addr(struct vty *vty, struct osmo_fd *ofd)
-{
-	char *name;
-	if (!ofd || ofd->fd < 0
-	|| !(name = osmo_sock_get_name(vty, ofd->fd))) {
-	    vty_out(vty, "(no addr)");
-	    return;
-	}
-	vty_out(vty, "%s", name);
-	talloc_free(name);
-}
-
 static void vty_dump_hnb_info__map_states(struct vty *vty, const char *name, unsigned int count,
 					  unsigned int state_count[])
 {
@@ -208,7 +196,7 @@ static void vty_dump_hnb_info(struct vty *vty, struct hnb_context *hnb)
 	unsigned long long sec;
 
 	vty_out(vty, "HNB ");
-	vty_out_ofd_addr(vty, hnb->conn? osmo_stream_srv_get_ofd(hnb->conn) : NULL);
+	vty_out(vty, "%s", hnb->conn ? osmo_stream_srv_get_sockname(hnb->conn) : "(no addr)");
 	vty_out(vty, " \"%s\"%s", hnb->identity_info, VTY_NEWLINE);
 	vty_out(vty, "    MCC %s MNC %s LAC %u RAC %u SAC %u CID %u SCTP-stream:HNBAP=%u,RUA=%u%s",
 		osmo_mcc_name(hnb->id.plmn.mcc), osmo_mnc_name(hnb->id.plmn.mnc, hnb->id.plmn.mnc_3_digits),
@@ -229,12 +217,6 @@ static void vty_dump_hnb_info(struct vty *vty, struct hnb_context *hnb)
 	}
 }
 
-static void vty_dump_ue_info(struct vty *vty, struct ue_context *ue)
-{
-	vty_out(vty, "UE IMSI \"%s\" context ID %u HNB %s%s", ue->imsi, ue->context_id,
-		hnb_context_name(ue->hnb), VTY_NEWLINE);
-}
-
 #define SHOW_HNB_STR SHOW_STR "Display information about HNB\n"
 
 DEFUN(show_hnb, show_hnb_cmd, "show hnb all",
@@ -279,18 +261,6 @@ DEFUN(show_one_hnb, show_one_hnb_cmd, "show hnb NAME ",
 	return CMD_SUCCESS;
 }
 
-DEFUN(show_ue, show_ue_cmd, "show ue all",
-      SHOW_STR "Display HNBAP information about UE\n" "All UE\n")
-{
-	struct ue_context *ue;
-
-	llist_for_each_entry(ue, &g_hnbgw->ue_list, list) {
-		vty_dump_ue_info(vty, ue);
-	}
-
-	return CMD_SUCCESS;
-}
-
 DEFUN(show_talloc, show_talloc_cmd, "show talloc", SHOW_STR "Display talloc info")
 {
 	talloc_report_full(g_hnbgw, stderr);
@@ -352,7 +322,7 @@ DEFUN(cfg_hnbgw_iuh_hnbap_allow_tmsi, cfg_hnbgw_iuh_hnbap_allow_tmsi_cmd,
       "hnbap-allow-tmsi (0|1)",
       "Allow HNBAP UE Register messages with TMSI or PTMSI identity\n"
       "Only accept IMSI identity, reject TMSI or PTMSI\n"
-      "Accept IMSI, TMSI or PTMSI as UE identity\n")
+      "Accept IMSI, TMSI or PTMSI as UE identity (default)\n")
 {
 	g_hnbgw->config.hnbap_allow_tmsi = (*argv[0] == '1');
 	return CMD_SUCCESS;
@@ -527,7 +497,7 @@ DEFUN(cfg_msc_nr, cfg_msc_nr_cmd,
       "Configure an IuCS link to an MSC\n"
       "MSC nr\n")
 {
-	return cnlink_nr(vty, &g_hnbgw->sccp.cnpool_iucs, argc, argv);
+	return cnlink_nr(vty, g_hnbgw->sccp.cnpool_iucs, argc, argv);
 }
 
 /* 'sgsn 0' */
@@ -536,12 +506,24 @@ DEFUN(cfg_sgsn_nr, cfg_sgsn_nr_cmd,
       "Configure an IuPS link to an SGSN\n"
       "SGSN nr\n")
 {
-	return cnlink_nr(vty, &g_hnbgw->sccp.cnpool_iups, argc, argv);
+	return cnlink_nr(vty, g_hnbgw->sccp.cnpool_iups, argc, argv);
 }
 
 /* 'msc 0'  / 'remote-addr my-msc'  and
  * 'sgsn 0' / 'remote-addr my-sgsn'
  */
+DEFUN(cfg_cnlink_name,
+      cfg_cnlink_name_cmd,
+      "name NAME",
+      "Set user defined name for this msc/sgsn\n"
+      "The user defined name to be set for this msc/sgsn\n")
+{
+	struct hnbgw_cnlink *cnlink = vty->index;
+	if (hnbgw_cnlink_set_name(cnlink, argv[0]) < 0)
+		return CMD_WARNING;
+	return CMD_SUCCESS;
+}
+
 DEFUN(cfg_cnlink_remote_addr,
       cfg_cnlink_remote_addr_cmd,
       "remote-addr NAME",
@@ -732,14 +714,14 @@ DEFUN(show_nri, show_nri_cmd,
 	 *   nri null add ...
 	 */
 	vty_out(vty, "hnbgw%s", VTY_NEWLINE);
-	cnpool_write_nri(vty, &g_hnbgw->sccp.cnpool_iucs, true);
-	cnpool_write_nri(vty, &g_hnbgw->sccp.cnpool_iups, true);
+	cnpool_write_nri(vty, g_hnbgw->sccp.cnpool_iucs, true);
+	cnpool_write_nri(vty, g_hnbgw->sccp.cnpool_iups, true);
 
 	/* msc 0
 	 *   nri add ...
 	 */
-	cnlinks_write_nri(vty, &g_hnbgw->sccp.cnpool_iucs, true);
-	cnlinks_write_nri(vty, &g_hnbgw->sccp.cnpool_iups, true);
+	cnlinks_write_nri(vty, g_hnbgw->sccp.cnpool_iucs, true);
+	cnlinks_write_nri(vty, g_hnbgw->sccp.cnpool_iups, true);
 	return CMD_SUCCESS;
 }
 
@@ -755,9 +737,9 @@ DEFUN_HIDDEN(cnpool_roundrobin_next, cnpool_roundrobin_next_cmd,
 {
 	struct hnbgw_cnpool *cnpool;
 	if (!strcmp("msc", argv[0]))
-		cnpool = &g_hnbgw->sccp.cnpool_iucs;
+		cnpool = g_hnbgw->sccp.cnpool_iucs;
 	else
-		cnpool = &g_hnbgw->sccp.cnpool_iups;
+		cnpool = g_hnbgw->sccp.cnpool_iups;
 	cnpool->round_robin_next_nr = atoi(argv[1]);
 	return CMD_SUCCESS;
 }
@@ -776,9 +758,9 @@ DEFUN(cnlink_ranap_reset, cnlink_ranap_reset_cmd,
 	int nr = atoi(argv[1]);
 
 	if (!strcmp("msc", msc_sgsn))
-		cnpool = &g_hnbgw->sccp.cnpool_iucs;
+		cnpool = g_hnbgw->sccp.cnpool_iucs;
 	else
-		cnpool = &g_hnbgw->sccp.cnpool_iups;
+		cnpool = g_hnbgw->sccp.cnpool_iups;
 
 	cnlink = cnlink_get_nr(cnpool, nr, false);
 	if (!cnlink) {
@@ -817,11 +799,11 @@ DEFUN(cfg_config_apply_sccp, cfg_config_apply_sccp_cmd,
 {
 	struct hnbgw_cnpool *cnpool;
 
-	cnpool = &g_hnbgw->sccp.cnpool_iucs;
+	cnpool = g_hnbgw->sccp.cnpool_iucs;
 	hnbgw_cnpool_apply_cfg(cnpool);
 	hnbgw_cnpool_cnlinks_start_or_restart(cnpool);
 
-	cnpool = &g_hnbgw->sccp.cnpool_iups;
+	cnpool = g_hnbgw->sccp.cnpool_iups;
 	hnbgw_cnpool_apply_cfg(cnpool);
 	hnbgw_cnpool_cnlinks_start_or_restart(cnpool);
 
@@ -1030,8 +1012,8 @@ static int config_write_hnbgw(struct vty *vty)
 	llist_for_each_entry(hnbp, &g_hnbgw->hnb_persistent_list, list)
 		write_one_hnbp(vty, hnbp);
 
-	_config_write_cnpool(vty, &g_hnbgw->sccp.cnpool_iucs);
-	_config_write_cnpool(vty, &g_hnbgw->sccp.cnpool_iups);
+	_config_write_cnpool(vty, g_hnbgw->sccp.cnpool_iucs);
+	_config_write_cnpool(vty, g_hnbgw->sccp.cnpool_iups);
 
 	if (g_hnbgw->config.nft_kpi.enable)
 		vty_out(vty, " nft-kpi%s%s%s",
@@ -1057,8 +1039,8 @@ static int config_write_hnbgw_iuh(struct vty *vty)
 	if (port && port != IUH_DEFAULT_SCTP_PORT)
 		vty_out(vty, "  local-port %u%s", port, VTY_NEWLINE);
 
-	if (g_hnbgw->config.hnbap_allow_tmsi)
-		vty_out(vty, "  hnbap-allow-tmsi 1%s", VTY_NEWLINE);
+	if (!g_hnbgw->config.hnbap_allow_tmsi)
+		vty_out(vty, "  hnbap-allow-tmsi 0%s", VTY_NEWLINE);
 
 	return CMD_SUCCESS;
 }
@@ -1073,6 +1055,7 @@ static void _config_write_cnlink(struct vty *vty, struct hnbgw_cnpool *cnpool)
 
 	llist_for_each_entry(cnlink, &cnpool->cnlinks, entry) {
 		vty_out(vty, "%s %d%s", cnpool->peer_name, cnlink->nr, VTY_NEWLINE);
+		vty_out(vty, " name %s%s", cnlink->name, VTY_NEWLINE);
 		if (cnlink->vty.remote_addr_name)
 			vty_out(vty, " remote-addr %s%s", cnlink->vty.remote_addr_name, VTY_NEWLINE);
 		cnlink_write_nri(vty, cnlink, false);
@@ -1081,13 +1064,13 @@ static void _config_write_cnlink(struct vty *vty, struct hnbgw_cnpool *cnpool)
 
 static int config_write_msc(struct vty *vty)
 {
-	_config_write_cnlink(vty, &g_hnbgw->sccp.cnpool_iucs);
+	_config_write_cnlink(vty, g_hnbgw->sccp.cnpool_iucs);
 	return CMD_SUCCESS;
 }
 
 static int config_write_sgsn(struct vty *vty)
 {
-	_config_write_cnlink(vty, &g_hnbgw->sccp.cnpool_iups);
+	_config_write_cnlink(vty, g_hnbgw->sccp.cnpool_iups);
 	return CMD_SUCCESS;
 }
 
@@ -1114,6 +1097,7 @@ static int config_write_hnbgw_pfcp(struct vty *vty)
 
 static void install_cnlink_elements(int node)
 {
+	install_element(node, &cfg_cnlink_name_cmd);
 	install_element(node, &cfg_cnlink_remote_addr_cmd);
 	install_element(node, &cfg_cnlink_nri_add_cmd);
 	install_element(node, &cfg_cnlink_nri_del_cmd);
@@ -1169,7 +1153,6 @@ void hnbgw_vty_init(void)
 	install_element_ve(&show_cnlink_cmd);
 	install_element_ve(&show_hnb_cmd);
 	install_element_ve(&show_one_hnb_cmd);
-	install_element_ve(&show_ue_cmd);
 	install_element_ve(&show_talloc_cmd);
 
 	install_element(HNBGW_NODE, &cfg_hnbgw_mgcp_cmd);
